@@ -1,6 +1,6 @@
 package Plack::Middleware::TemplateToolkit;
-BEGIN {
-  $Plack::Middleware::TemplateToolkit::VERSION = '0.2';
+{
+  $Plack::Middleware::TemplateToolkit::VERSION = '0.25';
 }
 # ABSTRACT: Serve files with Template Toolkit and Plack
 
@@ -8,12 +8,15 @@ use strict;
 use warnings;
 use 5.008_001;
 
+use parent 'Exporter'; # for 'use Plack::Middleware::TemplateToolkit $version;'
 use parent 'Plack::Middleware';
 use Plack::Request 0.994;
 use Plack::MIME;
 use Template 2;
 use Scalar::Util qw(blessed);
 use HTTP::Status qw(status_message);
+use Time::HiRes;
+use Plack::Middleware::Debug::Timer;
 use Encode;
 use Carp;
 
@@ -61,7 +64,7 @@ BEGIN {
 }
 
 use Plack::Util::Accessor (
-    qw(dir_index path extension content_type default_type tt root
+    qw(dir_index path extension content_type default_type tt root timer
         pass_through decode_request encode_response vars request_vars),
     @TT_CONFIG
 );
@@ -106,18 +109,28 @@ sub prepare_app {
 sub call {    # adapted from Plack::Middleware::Static
     my ( $self, $env ) = @_;
 
+    my $start = [ Time::HiRes::gettimeofday ] if $self->timer;
     my $res = $self->_handle_template($env);
-    if ( $res && not( $self->pass_through and $res->[0] == 404 ) ) {
-        return $res;
+
+    if ( !$res or ( $self->pass_through and $res->[0] == 404 ) ) {
+        if ( $self->app ) {
+            # pass to the next middleware/app
+            $res = $self->app->($env);
+            # if ( $self->catch_errors and $res->[0] =~ /^[45]/ ) {
+            # TODO: process error message (but better use callback)
+            # }
+        } else {
+            my $req = Plack::Request->new($env);
+            $res = $self->process_error( 404, 'Not found', 'text/plain', $req );
+        }
     }
 
-    if ( $self->app ) {
-        $res = $self->app->($env);
-
-        # TODO: if $res->[0] ne 200 and catch_errors: process error message?
-    } else {
-        my $req = Plack::Request->new($env);
-        $res = $self->process_error( 404, 'Not found', 'text/plain', $req );
+    if ($self->timer) {
+        my $end = [ Time::HiRes::gettimeofday ];
+        $env->{'tt.start'} = Plack::Middleware::Debug::Timer->format_time($start);
+        $env->{'tt.end'}   = Plack::Middleware::Debug::Timer->format_time($end);
+        $env->{'tt.elapsed'} 
+            = sprintf '%.6f s', Time::HiRes::tv_interval $start, $end;
     }
 
     $res;
@@ -157,9 +170,10 @@ sub process_error {
 
     $req = Plack::Request->new( { 'tt.vars' => {} } )
         unless blessed $req && $req->isa('Plack::Request');
-    $self->_set_vars($req);
+    eval { $self->_set_vars($req); };
 
     $req->env->{'tt.vars'}->{'error'} = $error;
+    $req->env->{'tt.vars'}->{'path'}  = $req->path_info;
     my $tpl = $self->{$code};
     my $res = $self->process_template( $tpl, $code, $req->env->{'tt.vars'} );
 
@@ -186,7 +200,6 @@ sub _set_vars {
 
     # we must not copy the vars by reference because
     # otherwise we might modify the same object
-    # TODO: catch error if $self->vars does not return hash ref or dies?
     my (%vars) = %{ $self->vars->($req) } if defined $self->vars;
 
     my $rv = $self->request_vars;
@@ -224,7 +237,6 @@ sub _set_vars {
     }
 
     if ( $env->{'tt.vars'} ) {
-
         # add to existing vars
         foreach ( keys %vars ) {
             $env->{'tt.vars'}->{$_} = $vars{$_};
@@ -238,40 +250,53 @@ sub _set_vars {
 sub _handle_template {
     my ( $self, $env ) = @_;
 
-    my $path       = $env->{PATH_INFO} || '/';
-    my $path_match = $self->path       || '/';
+    if ( not $env->{'tt.template'} ) {
+        my $path = $env->{'tt.path'} || do {
+           $env->{'tt.path'} = $env->{PATH_INFO} || '/';
+        };
 
-    for ($path) {
-        my $matched
-            = 'CODE' eq ref $path_match
-            ? $path_match->($_)
-            : $_ =~ $path_match;
-        return unless $matched;
+        my $path_match = $self->path || '/';
+        for ($path) {
+            my $matched
+                = 'CODE' eq ref $path_match
+                ? $path_match->($_)
+                : $_ =~ $path_match;
+            if (not $matched) {
+                delete $env->{'tt.path'};
+                return;
+            }
+        }
+
+        $path .= $self->dir_index if $path =~ /\/$/;
+        $path =~ s{^/}{};    # Do not want to enable absolute paths
+
+        my $extension = $self->extension;
+        if ( $extension and $path !~ /${extension}$/ ) {
+            # This 404 will be catched in method call if pass_through is set
+            my ($res, $tpl) = $self->process_error(
+                404, 'Not found', 'text/plain', Plack::Request->new($env) );
+            $env->{'tt.template'} = $tpl;
+            return $res;
+        }
+
+        $env->{'tt.template'} = $path;
+    } else {
+        delete $env->{'tt.path'};
     }
 
+    my ($res, $tpl);
     my $req = Plack::Request->new($env);
-
-    $env->{'tt.path'} = $req->path_info;
-
-    $path = $req->path;
-    $path .= $self->dir_index if $path =~ /\/$/;
-    $path =~ s{^/}{};    # Do not want to enable absolute paths
-
-    my $extension = $self->extension;
-    if ( $extension and $path !~ /${extension}$/ ) {
-
-        # TODO: we may want another code (forbidden) and message here
-        my ( $res, $tpl )
-            = $self->process_error( 404, 'Not found', 'text/plain', $req );
+    eval { $self->_set_vars($req); };
+    if ( $@ ) {
+        my $error = "error setting template variables: $@";
+        my $type = $self->content_type || $self->default_type;
+        ( $res, $tpl ) = $self->process_error( 500, $error, $type, $req );
         $env->{'tt.template'} = $tpl;
-        return $res;
+    } else {
+        $res = $self->process_template(
+            $env->{'tt.template'}, 200, $env->{'tt.vars'} );
     }
 
-    $self->_set_vars($req);
-
-    my $tpl = $path;
-    my $res = $self->process_template( $path, 200, $env->{'tt.vars'} );
-
     unless ( ref $res ) {
         my $type = $self->content_type || $self->default_type;
         if ( $res =~ /file error .+ not found/ ) {
@@ -282,10 +307,9 @@ sub _handle_template {
             }
             ( $res, $tpl ) = $self->process_error( 500, $res, $type, $req );
         }
+        $env->{'tt.template'} = $tpl;
     }
 
-    $env->{'tt.template'} = $tpl;    # not fully covered by unit tests
-
     return $res;
 }
 
@@ -307,7 +331,7 @@ __END__
 
         # These files can be served directly
         enable 'Static',
-            path => qr{\.[gif|png|jpg|swf|ico|mov|mp3|pdf|js|css]$},
+            path => qr{\.(gif|png|jpg|swf|ico|mov|mp3|pdf|js|css)$},
             root => $root;
 
         enable 'TemplateToolkit',
@@ -327,8 +351,8 @@ A minimal L<.psgi|PSGI> script as stand-alone application:
 Enable this middleware or application to allow your Plack-based application to
 serve files processed through L<Template Toolkit|Template> (TT). The idea
 behind this module is to provide content that is ALMOST static, but where
-having the power of TT can make the content easier to manage. You probably 
-only want to use this for the simpliest of sites, but it should be easy 
+having the power of TT can make the content easier to manage. You probably
+only want to use this for the simpliest of sites, but it should be easy
 enough to migrate to something more significant later.
 
 As L<Plack::Middleware> derives from L<Plack::Component> you can also use
@@ -338,7 +362,7 @@ Toolkit, treat this module as if it was called Plack::App::TemplateToolkit.
 You can mix this middleware with other Plack::App applications and
 Plack::Middleware which you will find on CPAN.
 
-This middleware reads and sets the PSGI environment variable tt.vars for
+This middleware reads and sets the PSGI environment variable C<tt.vars> for
 variables passed to templates. By default, the QUERY_STRING params are
 available to the templates, but the more you use these the harder it could be
 to migrate later so you might want to look at a propper framework such as
@@ -347,6 +371,8 @@ L<Catalyst> if you do want to use them:
   [% params.get('field') %] params is a L<Hash::MultiValue>
   [% request.parameters.field %] configured with request_vars => ['parameters']
 
+A full example application is included in this module in the exmple directory.
+
 =head1 CONFIGURATIONS
 
 You can use all configuration options that are supported by Template Toolkit
@@ -366,18 +392,23 @@ L<Plack::Builder> to map requests based on a path to this middleware.
 
 =item extension
 
-Limit to only files with this extension. Requests for other files will result in
-a 404 response or be passed to the next application if pass_through is set.
+Limit to only files with this extension. Requests for other files within
+C<path> will result in a 404 response or be passed to the next application if
+C<pass_through> is set.
 
 =item content_type
 
 Specify the Content-Type header you want returned. If not specified, the
 content type will be guessed by L<Plack::MIME> based on the file extension
-with default_type as default.
+with C<default_type> as default.
 
 =item default_type
 
-Specify the default Content-Type header. Defaults to to text/html.
+Specify the default Content-Type header. Defaults to to C<text/html>.
+
+=item dir_index
+
+Which file to use as a directory index, defaults to C<index.html>.
 
 =item vars
 
@@ -389,19 +420,15 @@ variables in the tt.vars environment variable.
 
 =item request_vars
 
-Specify a list of request variables from L<Plack::Request> to be collected
-in a template variable 'request'. For instance C<<[path base]>> gives you
-the template variables C<request.path> and C<request.base>. Setting this
-parameter to 'all' gives you the original Plack::Request object, but this
-is unstable, bad practice because the object may change and your templates
-may damage the request object.
+Specify a list of request variables from L<Plack::Request> to be collected in a
+template variable 'request'. For instance C< ['path','base'] > gives you the
+template variables C<request.path> and C<request.base>. Setting this parameter
+to 'all' gives you the original Plack::Request object, but this is unstable,
+bad practice because the object may change and your templates may damage the
+request object.
 
 By default the request variables are decoded from byte strings to Unicode.
-You can change this with the configuration value 'decode_request'.
-
-=item dir_index
-
-Which file to use as a directory index, defaults to index.html
+You can change this with the configuration value C<decode_request>.
 
 =item pass_through
 
@@ -425,8 +452,8 @@ Directly set an instance of L<Template> instead of creating a new one:
 
 If your templates or template variables are Unicode strings, the output must be
 encoded, because PSGI expects the content body to be a byte stream. You can
-specify an encoding, such as 'utf8' with this parameter, so the output is
-encoded to a byte string. The default setting is 'utf8' which encodes to UTF-8
+specify an encoding, such as C<utf8> with this parameter, so the output is
+encoded to a byte string. The default setting is C<utf8> which encodes to UTF-8
 bytes.  This default option is useful if your input contains non-ASCII
 characters, but it may lead to double encoded UTF-8 bytes, if you accidently
 mix strings with UTF-8 flag and without.  To find such implicit encoding
@@ -434,27 +461,30 @@ conversions, try L<encoding::warnings>.
 
 =item decode_request
 
-Similar to 'encode_response', this parameter decodes the input request from a
-byte string to an encoding of your choice. Set to 'utf8' by default.
+Similar to C<encode_response>, this parameter decodes the input request from a
+byte string to an encoding of your choice. Set to C<utf8> by default.
 
 It is highly recommended to use L<Plack::Middleware::Lint> and test your app
 with Unicode from several sources (templates, variables, parameters, ...).
 
+=item timer
+
+Time the processing and add C<tt.start>, C<tt.end>, and C<tt.elapsed> to the
+environment.
+
+=item 404 and 500
+
+Specifies an error template that is processed when a file was not found (404)
+or on server error (500). The template variables C<error> with an error message
+and C<path> with the request path are set for processing. If an error template
+count not be found and processed, another error with status code 500 is
+returned, possibly also as template.
+
 =back
 
-In addition you can specify templates for error codes, for instance:
-
-  Plack::Middleware::TemplateToolkit->new(
-      INCLUDE_PATH => '/path/to/htdocs/',
-      404  => 'page_not_found.html' # = /path/to/htdocs/page_not_found.html
-  );
-
-If a specified error templates could not be found and processed, an error
-with HTTP status code 500 is returned, possibly also as template.
-
 =head1 ENVIRONMENT
 
-This middleware inspects and/or manipulates the following variables from 
+This middleware inspects and/or manipulates the following variables from
 the PSGI environment:
 
 =over 4
@@ -466,11 +496,14 @@ Injected as template variables if defined. Set to the template variables.
 =item tt.path
 
 Set to the template that was asked to process. This is equal to the local path
-(C<path_info> in L<Plack::Request>) if the request matched.
+(C<path_info> in L<Plack::Request>) if the request matched. If this variable is
+set I<before> the middleware is called, it uses its value instead of path_info.
 
 =item tt.template
 
-Set to the template that has actually been processed.
+Set to the template that has actually been processed. If this variable is set
+I<before> the middleware is called, the specified template is processed. In this
+all other settings (path, extensions, dir_index, and tt.path) are ignored.
 
 =back
 
@@ -492,7 +525,7 @@ On failure this method returns an error message instead of a reference.
 =head2 process_error ( $code, $error, $type, $req ) = @_;
 
 Returns a PSGI response to be used as error message. Error templates are used
-if they have been specified and prepare_app has been called before. This method 
+if they have been specified and prepare_app has been called before. This method
 tries hard not to fail: undefined parameters are replaced by default values.
 In list context this returns a PSGI response and the actual template that has
 been used to create the error document.
